@@ -2,8 +2,8 @@
 #define _CONFIG_H
 
 /* Minix release and version numbers. */
-#define OS_RELEASE "2.0"
-#define OS_VERSION "2"
+#define OS_RELEASE "2"
+#define OS_VERSION "0.3"
 
 /* This file sets configuration parameters for the MINIX kernel, FS, and MM.
  * It is divided up into two main sections.  The first section contains
@@ -28,16 +28,8 @@
 #define _WORD_SIZE	_EM_WSIZE
 #endif
 
-
-/* If ROBUST is set to 1, writes of i-node, directory, and indirect blocks
- * from the cache happen as soon as the blocks are modified.  This gives a more
- * robust, but slower, file system.  If it is set to 0, these blocks are not
- * given any special treatment, which may cause problems if the system crashes.
- */
-#define ROBUST             0	/* 0 for speed, 1 for robustness */
-
 /* Number of slots in the process table for user processes. */
-#define NR_PROCS          40
+#define NR_PROCS          32
 
 /* The buffer cache should be made as large as you can afford. */
 #if (MACHINE == IBM_PC && _WORD_SIZE == 2)
@@ -46,8 +38,8 @@
 #endif
 
 #if (MACHINE == IBM_PC && _WORD_SIZE == 4)
-#define NR_BUFS         1024	/* # blocks in the buffer cache */
-#define NR_BUF_HASH     2048	/* size of buf hash table; MUST BE POWER OF 2*/
+#define NR_BUFS           80	/* # blocks in the buffer cache */
+#define NR_BUF_HASH      128	/* size of buf hash table; MUST BE POWER OF 2*/
 #endif
 
 #if (MACHINE == SUN_4_60)
@@ -55,101 +47,50 @@
 #define NR_BUF_HASH	 512	/* size of buf hash table; MUST BE POWER OF 2*/
 #endif
 
-#if (MACHINE == ATARI)
-#define NR_BUFS		1536	/* # blocks in the buffer cache (<=1536) */
-#define NR_BUF_HASH	2048	/* size of buf hash table; MUST BE POWER OF 2*/
-#endif
-
 /* Defines for kernel configuration. */
 #define AUTO_BIOS          0	/* xt_wini.c - use Western's autoconfig BIOS */
 #define LINEWRAP           1	/* console.c - wrap lines at column 80 */
 #define ALLOW_GAP_MESSAGES 1	/* proc.c - allow messages in the gap between
 				 * the end of bss and lowest stack address */
 
+/* Number of controller tasks (/dev/cN device classes). */
+#define NR_CTRLRS          2
+
 /* Enable or disable the second level file system cache on the RAM disk. */
 #define ENABLE_CACHE2      1
 
 /* Include or exclude device drivers.  Set to 1 to include, 0 to exclude. */
 #define ENABLE_AT_WINI     1	/* enable AT winchester driver */
+#define   ENABLE_ATAPI     1	/* add ATAPI support to AT driver */
 #define ENABLE_BIOS_WINI   1	/* enable BIOS winchester driver */
-#define ENABLE_ESDI_WINI   0	/* enable ESDI winchester driver */
+#define ENABLE_ESDI_WINI   1	/* enable ESDI winchester driver */
 #define ENABLE_XT_WINI     0	/* enable XT winchester driver */
 #define ENABLE_AHA1540_SCSI 1	/* enable Adaptec 1540 SCSI driver */
-#define ENABLE_MITSUMI_CDROM 0	/* enable Mitsumi CD-ROM driver */
-#define ENABLE_DOSFAT      1	/* enable DOS FAT file virtual disk driver */
+#define ENABLE_FATFILE     1	/* enable FAT file virtual disk driver */
 #define ENABLE_DOSFILE     1	/* enable DOS file virtual disk driver */
-#define ENABLE_SB_AUDIO    0	/* enable Soundblaster audio driver */
+#define ENABLE_SB16        0	/* enable Soundblaster audio driver */
+#define ENABLE_PRINTER     0	/* enable printer driver */
+#define ENABLE_USERBIOS    0	/* enable user mode BIOS calls */
 
 /* DMA_SECTORS may be increased to speed up DMA based drivers. */
-#define DMA_SECTORS       16	/* DMA buffer size (must be >= 1) */
+#define DMA_SECTORS        1	/* DMA buffer size (must be >= 1) */
 
-/* Enable or disable networking code (TCP/IP task & drivers). */
-#define ENABLE_NETWORKING  1	/* enable TCP/IP code (main switch) */
-#define ENABLE_WDETH       0	/* enable Western Digital WD80x3 */
-#define ENABLE_NE2000      1	/* enable Novell NE1000/NE2000 */
-#define ENABLE_3C503       0	/* enable 3Com Etherlink II (3C503) */
+/* Enable or disable networking drivers. */
+#define ENABLE_DP8390      0	/* enable DP8390 ethernet driver */
+#define   ENABLE_WDETH     1	/* add Western Digital WD80x3 to DP8390 */
+#define   ENABLE_NE2000    1	/* add Novell NE1000/NE2000 to DP8390 */
+#define   ENABLE_3C503     1	/* add 3Com Etherlink II (3C503) to DP8390 */
 
 /* Include or exclude backwards compatibility code. */
 #define ENABLE_BINCOMPAT   0	/* for binaries using obsolete calls */
 #define ENABLE_SRCCOMPAT   0	/* for sources using obsolete calls */
 
-/* Determine which device to use for pipes. */
-#define PIPE_DEV    ROOT_DEV	/* put pipes on root device */
-
 /* NR_CONS, NR_RS_LINES, and NR_PTYS determine the number of terminals the
  * system can handle.
  */
 #define NR_CONS            2	/* # system consoles (1 to 8) */
-#define	NR_RS_LINES	   2	/* # rs232 terminals (0, 1, or 2) */
-#define	NR_PTYS		   4	/* # pseudo terminals (0 to 64) */
-
-#if (MACHINE == ATARI)
-/* The next define says if you have an ATARI ST or TT */
-#define ATARI_TYPE	  TT
-#define ST		   1	/* all ST's and Mega ST's */
-#define STE		   2	/* all STe and Mega STe's */
-#define TT		   3
-
-/* if SCREEN is set to 1 graphical screen operations are possible */
-#define SCREEN             1	
-
-/* This define says whether the keyboard generates VT100 or IBM_PC escapes. */
-#define KEYBOARD       VT100	/* either VT100 or IBM_PC */
-#define VT100		 100
-
-/* The next define determines the kind of partitioning. */
-#define PARTITIONING   SUPRA	/* one of the following or ATARI */
-#define SUPRA		   1	/*ICD, SUPRA and BMS are all the same */
-#define BMS		   1
-#define ICD		   1
-#define CBHD		   2
-#define EICKMANN	   3
-
-/* Define the number of hard disk drives on your system. */
-#define NR_ACSI_DRIVES	   3	/* typically 0 or 1 */
-#define NR_SCSI_DRIVES	   1	/* typically 0 (ST, STe) or 1 (TT) */
-
-/* Some systems need to have a little delay after each winchester
- * commands. These systems need FAST_DISK set to 0. Other disks do not
- * need this delay, and thus can have FAST_DISK set to 1 to avoid this delay.
- */
-#define FAST_DISK	   1	/* 0 or 1 */
-
-/* Note: if you want to make your kernel smaller, you can set NR_FD_DRIVES
- * to 0. You will still be able to boot minix.img from floppy. However, you
- * MUST fetch both the root and usr filesystem from a hard disk
- */
-
-/* Define the number of floppy disk drives on your system. */
-#define NR_FD_DRIVES	   1	/* 0, 1, 2 */
-
-/* This configuration define controls parallel printer code. */
-#define PAR_PRINTER	   1	/* disable (0) / enable (1) parallel printer */
-
-/* This configuration define controls disk controller clock code. */
-#define HD_CLOCK	   1	/* disable (0) / enable (1) hard disk clock */
-
-#endif
+#define	NR_RS_LINES	   0	/* # rs232 terminals (0, 1, or 2) */
+#define	NR_PTYS		   0	/* # pseudo terminals (0 to 64) */
 
 
 /*===========================================================================*
@@ -169,24 +110,15 @@
 
 #if (MACHINE == IBM_PC)
 #define CHIP          INTEL
-#define SHADOWING	  0
-#define ENABLE_WINI	(ENABLE_AT_WINI || ENABLE_BIOS_WINI || \
-			ENABLE_ESDI_WINI || ENABLE_XT_WINI)
-#define ENABLE_SCSI	(ENABLE_AHA1540_SCSI)
-#define ENABLE_CDROM	(ENABLE_MITSUMI_CDROM)
-#define ENABLE_AUDIO	(ENABLE_SB_AUDIO)
-#define ENABLE_DOSDSK	(ENABLE_DOSFAT || ENABLE_DOSFILE)
 #endif
 
 #if (MACHINE == ATARI) || (MACHINE == AMIGA) || (MACHINE == MACINTOSH)
 #define CHIP         M68000
-#define SHADOWING	  1
 #endif
 
 #if (MACHINE == SUN_4) || (MACHINE == SUN_4_60)
 #define CHIP          SPARC
 #define FP_FORMAT   FP_IEEE
-#define SHADOWING	  0
 #endif
 
 #if (MACHINE == ATARI) || (MACHINE == SUN_4)
@@ -196,21 +128,12 @@
 
 #if (ATARI_TYPE == TT) /* and all other 68030's */
 #define FPP
-#undef SHADOWING
-#define SHADOWING 0
 #endif
 
 #ifndef FP_FORMAT
 #define FP_FORMAT   FP_NONE
 #endif
 
-/* The file buf.h uses MAYBE_WRITE_IMMED. */
-#if ROBUST
-#define MAYBE_WRITE_IMMED  WRITE_IMMED	/* slower but perhaps safer */
-#else
-#define MAYBE_WRITE_IMMED 0		/* faster */
-#endif
-
 #ifndef MACHINE
 error "In <minix/config.h> please define MACHINE"
 #endif
