@@ -21,6 +21,7 @@
 #define IRREL		"/usr/lib/irrel"
 #define CEM		"/usr/lib/ncem"
 #define M2EM		"/usr/lib/nm2em"
+#define ENCODE		"/usr/lib/em_encode"
 #define OPT		"/usr/lib/nopt"
 #define CG		"/usr/lib/ncg"
 #define AS		"/usr/lib/as"
@@ -65,10 +66,11 @@ struct passinfo {
 #define O_OUTPUT 0100		/* -o outputfile, hack for as */
 #define PREPALWAYS	0200	/* always to be preprocessed */
 #define PREPCOND	0400	/* preprocessed when starting with '#' */
+#define PREPNOLN	01000	/* suppress line number info (cpp -P) */
 };
 
-#define MAXHEAD	6
-#define MAXTAIL	6
+#define MAXHEAD	10
+#define MAXTAIL	5
 #define MAXPASS	7
 
 /*	Every language handled by this program has a "compile" structure
@@ -97,10 +99,11 @@ struct compile {
 
 struct passinfo passinfo[] = {
 	{ "cpp", PP, "CPP", "i", "wo=o,I*,D*,U*,P", INPUT|STDOUT },
-	{ "irrel", IRREL, "i", "i", "", INPUT},
-	{ "cem", CEM, "i,c", "k", "m,p,wa=a,wo=o,ws=s,w,T*", INPUT|OUTPUT|PREPALWAYS },
+	{ "irrel", IRREL, "i", "i", "m", INPUT},
+	{ "cem", CEM, "i,c", "k", "m=o,p,wa=a,wo=o,ws=s,w,T*", INPUT|OUTPUT|PREPALWAYS },
 	{ "pc", PEM, "i,p", "k", "n=L,w,a,A,R", INPUT|OUTPUT|PREPCOND },
 	{ "m2", M2EM, "i,mod", "k", "n=L,w*,A,R,W*,3,I*", INPUT|OUTPUT|PREPCOND },
+	{ "encode", ENCODE, "i,e", "k", "", INPUT|STDOUT|PREPCOND|PREPNOLN },
 	{ "opt", OPT, "k", "m", "", STDIN|STDOUT },
 	{ "cg", CG, "m", "s", "O=p4", INPUT|OUTPUT },
 	{ "as", AS, "i,s", "o", "T*", INPUT|O_OUTPUT|PREPCOND },
@@ -109,14 +112,20 @@ struct passinfo passinfo[] = {
 	{ 0}
 };
 
-#define	PREP_FLAGS	"-D_EM_WSIZE=2", "-D_EM_PSIZE=2", "-D_EM_LSIZE=4" \
-			, "-D__ACK__", "-D__minix", "-D__i86",
+#define	PREP_FLAGS	"-D_EM_WSIZE=2", "-D_EM_PSIZE=2", "-D_EM_SSIZE=2", \
+			"-D_EM_LSIZE=4", "-D_EM_FSIZE=4", "-D_EM_DSIZE=8", \
+			"-D__ACK__", "-D__minix", "-D__i86"
 
 struct pass preprocessor = { "cpp",
 			    { PREP_FLAGS }
 			    , {0}
 			    };
 
+struct pass prepnoln = { "cpp",
+			    { PREP_FLAGS, "-P" }
+			    , {0}
+			    };
+
 struct pass irrel = { "irrel",
 			    {0}
 			};
@@ -160,6 +169,16 @@ struct compile passes[] = {
 	},
 	DEFLANG
 },
+{	"e", "encode",
+	{	{ "encode", {0}, {0}},
+		{ "opt", {0}, {0} },
+		{ "cg", {0}, {0} },
+		{ "as", {"-"}, {0} },
+		{ "ld", {0}, {"*", END}},
+		{ "cv", {0}, {0} }
+	},
+	DEFLANG
+},
 {	"s", "as",
 	{	{ "as", {0}, {0}}
 	},
@@ -197,12 +216,12 @@ char *ProgCall = 0;
 int RET_CODE = 0;
 
 char *stopsuffix;
-int m_flag = 0;
 int v_flag = 0;
 int t_flag = 0;
 int noexec = 0;
-int fp_lib = 0;
+int fp_lib = 1;
 int E_flag = 0;
+int i_flag = 1;
 
 
 USTRING curfil;
@@ -314,70 +333,82 @@ main(argc, argv)
 			continue;
 		}
 
-		switch (str[1]) {
+		if (strcmp(str, "-com") == 0) {
+			i_flag = 0;
+		} else
+		if (strcmp(str, "-sep") == 0) {
+			i_flag = 1;
+		} else {
+			switch (str[1]) {
+
+			case 'c':
+				stopsuffix = "o";
+				if (str[2] == '.') stopsuffix = str + 3;
+				break;
+			case 'f':
+				fp_lib = (strcmp(str+2, "hard") != 0);
+				break;
+			case 'F':
+			case 'W':
+				/* Ignore. */
+				break;
+			case 'L':
+				append(&LDIRS, &str[2]);
+				count = strlen(&str[2]);
+				if (count > maxLlen) maxLlen = count;
+				break;
+			case 'l':
+				append(&SRCFILES, library(&str[2]));
+				break;
+			case 'm':
+				/* Use -m, ignore -mxxx. */
+				if (str[2] == 0) append(&FLAGS, str);
+				break;
+			case 'o':
+				if (argc-- >= 0)
+					o_FILE = *argv++;
+				break;
+			case 'S':
+				stopsuffix = "s";
+				break;
+			case 'E':
+				E_flag = 1;
+				stopsuffix = "i";
+				break;
+			case 'P':
+				stopsuffix = "i";
+				append(&FLAGS, str);
+				break;
+			case 'v':
+				v_flag++;
+				if (str[2] == 'n')
+					noexec = 1;
+				break;
+			case 't':
+				/* save temporaries */
+				t_flag++;
+				break;
+			case '.':
+				if (str[2] == 'o') {
+					/* no runtime start-off */
+					loader->pp_head[0] = 0;
+				}
+				break;
+			case 'i':
+				i_flag++;
+				break;
+			case 'T':
+				tmpdir = &str[2];
+				/*FALLTHROUGH*/
+			default:
+				append(&FLAGS, str);
 
-		case 'c':
-			stopsuffix = "o";
-			if (str[2] == '.') stopsuffix = str + 3;
-			break;
-		case 'f':
-			fp_lib = 1;
-			break;
-		case 'F':
-		case 'W':
-			/* Ignore. */
-			break;
-		case 'L':
-			append(&LDIRS, &str[2]);
-			count = strlen(&str[2]);
-			if (count > maxLlen) maxLlen = count;
-			break;
-		case 'l':
-			append(&SRCFILES, library(&str[2]));
-			break;
-		case 'm':
-			if (str[2] == 0)
-				m_flag++;
-			break;
-		case 'o':
-			if (argc-- >= 0)
-				o_FILE = *argv++;
-			break;
-		case 'S':
-			stopsuffix = "s";
-			break;
-		case 'E':
-			E_flag = 1;
-			stopsuffix = "i";
-			break;
-		case 'P':
-			stopsuffix = "i";
-			append(&FLAGS, str);
-			break;
-		case 'v':
-			v_flag++;
-			if (str[2] == 'n')
-				noexec = 1;
-			break;
-		case 't':
-			/* save temporaries */
-			t_flag++;
-			break;
-		case '.':
-			if (str[2] == 'o') {
-				/* no runtime start-off */
-				loader->pp_head[0] = 0;
 			}
-			break;
-		case 'T':
-			tmpdir = &str[2];
-			/*FALLTHROUGH*/
-		default:
-			append(&FLAGS, str);
-
 		}
 	}
 
+	if (i_flag) append(&FLAGS, "-i");
+
 	mktempname(tmpname);
 
 	count = SRCFILES.al_argc;
@@ -762,16 +793,16 @@ apply(pinf, cp, name, passindex, noremove, first, resultname)
 	     )
 	   ) {
 		mkstr(newfil, tmpname, passinfo[0].p_to, "");
-		mkvec(call, name, newfil, &preprocessor, &passinfo[0]);
+		mkvec(call, name, newfil,
+			(pinf->p_flags & PREPNOLN) ? &prepnoln : &preprocessor,
+			&passinfo[0]);
 		if (! runvec(call, &passinfo[0], name, newfil)) {
 			cleanup(newfil);
 			return 0;
 		}
-		/*
-		 * When -m is specified and we have a .c file, then we
-		 * should run irrel.
-		 */
-		if (m_flag && cfile(name)) {
+
+		/* A .c file must always be mishandled by irrel. */
+		if (cfile(name)) {
 			/* newfil is OK */
 			mkvec(call, newfil, newfil, &irrel, &passinfo[1]);
 			if (! runvec(call, &passinfo[1], newfil, newfil)) {
