@@ -1,5 +1,7 @@
 /* time - time a command	Authors: Andy Tanenbaum & Michiel Huisjes */
 
+#define NEW	1
+
 #include <sys/types.h>
 #include <sys/times.h>
 #include <limits.h>
@@ -11,11 +13,16 @@
 #include <minix/minlib.h>
 #include <stdio.h>
 
+/* -DNEW prints time to 0.01 sec. */
+#ifdef NEW		
+#define HUNDREDTHS 1
+#endif
+
 char **args;
 char *name;
 
 int digit_seen;
-char a[] = "12:34:56.78";
+char a[] = "        . \0";
 
 _PROTOTYPE(int main, (int argc, char **argv));
 _PROTOTYPE(void print_time, (clock_t t));
@@ -29,7 +36,12 @@ char *argv[];
 
   struct tms pre_buf, post_buf;
   int status, pid;
+#if _VMD_EXT
+  struct timeval start_time, end_time;
+#else
   time_t start_time, end_time;
+#endif
+  clock_t real_time;
 
   if (argc == 1) exit(0);
 
@@ -37,7 +49,11 @@ char *argv[];
   name = argv[1];
 
   /* Get real time at start of run. */
+#if _VMD_EXT
+  (void) sysutime(UTIME_TIMEOFDAY, &start_time);
+#else
   (void) time(&start_time);
+#endif
 
   /* Fork off child. */
   if ((pid = fork()) < 0) {
@@ -53,22 +69,29 @@ char *argv[];
   do {
 	times(&pre_buf);
   } while (wait(&status) != pid);
+#if _VMD_EXT
+  (void) sysutime(UTIME_TIMEOFDAY, &end_time);
+  real_time = (end_time.tv_sec - start_time.tv_sec) * CLOCKS_PER_SEC
+	+ (end_time.tv_usec - start_time.tv_usec) * CLOCKS_PER_SEC / 1000000;
+#else
   (void) time(&end_time);
+  real_time = (end_time - start_time) * CLOCKS_PER_SEC;
+#endif
 
   if ((status & 0377) != 0) std_err("Command terminated abnormally.\n");
   times(&post_buf);
 
-  /* Print results.  -DNEW_TIME_FORMAT enables time on one line to 0.01 sec. */
-#ifndef NEW_TIME_FORMAT
+  /* Print results. -DNEW enables time on one line to 0.01 sec */
+#ifndef NEW
   std_err("real ");
-  print_time((clock_t) (end_time - start_time) * CLOCKS_PER_SEC);
+  print_time(real_time);
   std_err("\nuser ");
   print_time(post_buf.tms_cutime - pre_buf.tms_cutime);
   std_err("\nsys  ");
   print_time(post_buf.tms_cstime - pre_buf.tms_cstime);
   std_err("\n");
 #else
-  print_time((clock_t) (end_time - start_time) * CLOCKS_PER_SEC);
+  print_time(real_time);
   std_err(" real");
   print_time(post_buf.tms_cutime - pre_buf.tms_cutime);
   std_err(" user");
@@ -108,7 +131,9 @@ register clock_t t;
   else
 	a[7] = '0';
   a[9] = hundredths / 10 + '0';
+#ifdef HUNDREDTHS		/* tenths used to be enough */
   a[10] = hundredths % 10 + '0';
+#endif
   std_err(a);
 }
 
