@@ -58,10 +58,10 @@ EXTERN int bufs_in_use;		/* # bufs currently in use (not on free list)*/
 #define WRITE_IMMED        0100	/* block should be written to disk now */
 #define ONE_SHOT           0200	/* set if block not likely to be needed soon */
 
-#define INODE_BLOCK       (0 + MAYBE_WRITE_IMMED)	 /* inode block */
-#define DIRECTORY_BLOCK   (1 + MAYBE_WRITE_IMMED)	 /* directory block */
-#define INDIRECT_BLOCK    (2 + MAYBE_WRITE_IMMED)	 /* pointer block */
-#define MAP_BLOCK         (3 + MAYBE_WRITE_IMMED)	 /* bit map */
+#define INODE_BLOCK        0				 /* inode block */
+#define DIRECTORY_BLOCK    1				 /* directory block */
+#define INDIRECT_BLOCK     2				 /* pointer block */
+#define MAP_BLOCK          3				 /* bit map */
 #define ZUPER_BLOCK       (4 + WRITE_IMMED + ONE_SHOT)	 /* super block */
 #define FULL_DATA_BLOCK    5		 	 	 /* data, fully used */
 #define PARTIAL_DATA_BLOCK 6 				 /* data, partly used*/
