@@ -13,6 +13,7 @@ EXTERN int nr_locks;		/* number of locks currently in place */
 EXTERN int reviving;		/* number of pipe processes to be revived */
 EXTERN off_t rdahedpos;		/* position to read ahead */
 EXTERN struct inode *rdahed_inode;	/* pointer to inode to read ahead */
+EXTERN Dev_t root_dev;		/* device number of the root device */
 
 /* The parameters of the call are kept here. */
 EXTERN message m;		/* the input message itself */
@@ -25,8 +26,8 @@ EXTERN char user_path[PATH_MAX];/* storage for user path name */
 EXTERN int err_code;		/* temporary storage for error number */
 EXTERN int rdwt_err;		/* status of last disk i/o request */
 
-/* Data which need initialization. */
-extern _PROTOTYPE (int (*call_vector[]), (void) ); /* sys call table */
+/* Data initialized elsewhere. */
+extern _PROTOTYPE (int (*call_vec[]), (void) ); /* sys call table */
 extern int max_major;  /* maximum major device (+ 1) */
 extern char dot1[2];   /* dot1 (&dot1[0]) and dot2 (&dot2[0]) have a special */
 extern char dot2[3];   /* meaning to search_dir: no access permission check. */
