@@ -16,10 +16,11 @@ Copyright 1995 Philip Homburg
 typedef void (*arp_func_t) ARGS(( int fd, ipaddr_t ipaddr,
 	ether_addr_t *ethaddr ));
 
+void arp_prep ARGS(( void ));
 void arp_init ARGS(( void ));
-void arp_set_ipaddr ARGS(( int ip_port, ipaddr_t ipaddr ));
-int arp_set_cb ARGS(( int ip_port, int eth_port, arp_func_t arp_func ));
-int arp_ip_eth ARGS(( int ip_port, ipaddr_t ipaddr, ether_addr_t *ethaddr ));
+void arp_set_ipaddr ARGS(( int eth_port, ipaddr_t ipaddr ));
+int arp_set_cb ARGS(( int eth_port, int ip_port, arp_func_t arp_func ));
+int arp_ip_eth ARGS(( int eth_port, ipaddr_t ipaddr, ether_addr_t *ethaddr ));
 
 #endif /* ARP_H */
 
