@@ -2,12 +2,17 @@
 inet/mq.c
 
 Created:	Jan 3, 1992 by Philip Homburg
+
+Copyright 1995 Philip Homburg
 */
 
 #include "inet.h"
 #include "mq.h"
+#include "generic/assert.h"
 
-#define MQ_SIZE		64
+THIS_FILE
+
+#define MQ_SIZE		128
 
 PRIVATE mq_t mq_list[MQ_SIZE];
 PRIVATE mq_t *mq_freelist;
@@ -21,6 +26,7 @@ void mq_init()
 	{
 		mq_list[i].mq_next= mq_freelist;
 		mq_freelist= &mq_list[i];
+		mq_list[i].mq_allocated= 0;
 	}
 }
 
@@ -29,11 +35,12 @@ mq_t *mq_get()
 	mq_t *mq;
 
 	mq= mq_freelist;
-	if (mq)
-	{
-		mq_freelist= mq->mq_next;
-		mq->mq_next= NULL;
-	}
+	assert(mq != NULL);
+
+	mq_freelist= mq->mq_next;
+	mq->mq_next= NULL;
+	assert(mq->mq_allocated == 0);
+	mq->mq_allocated= 1;
 	return mq;
 }
 
@@ -42,4 +49,10 @@ mq_t *mq;
 {
 	mq->mq_next= mq_freelist;
 	mq_freelist= mq;
+	assert(mq->mq_allocated == 1);
+	mq->mq_allocated= 0;
 }
+
+/*
+ * $PchId: mq.c,v 1.6 1996/05/07 21:10:16 philip Exp $
+ */
