@@ -29,42 +29,46 @@
 #define KBIT		0x80	/* bit used to ack characters to keyboard */
 
 /* Miscellaneous. */
-#define ESC_SCAN	   1	/* Reboot key when panicking */
-#define SLASH_SCAN	  53	/* to recognize numeric slash */
-#define HOME_SCAN	  71	/* first key on the numeric keypad */
-#define DEL_SCAN	  83	/* DEL for use in CTRL-ALT-DEL reboot */
+#define ESC_SCAN	0x01	/* Reboot key when panicking */
+#define SLASH_SCAN	0x35	/* to recognize numeric slash */
+#define RSHIFT_SCAN	0x36	/* to distinguish left and right shift */
+#define HOME_SCAN	0x47	/* first key on the numeric keypad */
+#define INS_SCAN	0x52	/* INS for use in CTRL-ALT-INS reboot */
+#define DEL_SCAN	0x53	/* DEL for use in CTRL-ALT-DEL reboot */
 #define CONSOLE		   0	/* line number for console */
 #define MEMCHECK_ADR   0x472	/* address to stop memory check after reboot */
 #define MEMCHECK_MAG  0x1234	/* magic number to stop memory check */
 
-#define kb_addr()	(&kb_lines[0])	/* there is only one keyboard */
 #define KB_IN_BYTES	  32	/* size of keyboard input buffer */
 
-PRIVATE int alt1;		/* left alt key state */
-PRIVATE int alt2;		/* right alt key state */
-PRIVATE int capslock;		/* caps lock key state */
+PRIVATE char ibuf[KB_IN_BYTES];	/* input buffer */
+PRIVATE char *ihead = ibuf;	/* next free spot in input buffer */
+PRIVATE char *itail = ibuf;	/* scan code to return to TTY */
+PRIVATE int icount;		/* # codes in buffer */
+
 PRIVATE int esc;		/* escape scan code detected? */
-PRIVATE int control;		/* control key state */
-PRIVATE int caps_off;		/* 1 = normal position, 0 = depressed */
-PRIVATE int numlock;		/* number lock key state */
-PRIVATE int num_off;		/* 1 = normal position, 0 = depressed */
-PRIVATE int slock;		/* scroll lock key state */
-PRIVATE int slock_off;		/* 1 = normal position, 0 = depressed */
-PRIVATE int shift;		/* shift key state */
+PRIVATE int alt_l;		/* left alt key state */
+PRIVATE int alt_r;		/* right alt key state */
+PRIVATE int alt;		/* either alt key */
+PRIVATE int ctrl_l;		/* left control key state */
+PRIVATE int ctrl_r;		/* right control key state */
+PRIVATE int ctrl;		/* either control key */
+PRIVATE int shift_l;		/* left shift key state */
+PRIVATE int shift_r;		/* right shift key state */
+PRIVATE int shift;		/* either shift key */
+PRIVATE int num_down;		/* num lock key depressed */
+PRIVATE int caps_down;		/* caps lock key depressed */
+PRIVATE int scroll_down;	/* scroll lock key depressed */
+PRIVATE int locks[NR_CONS];	/* per console lock keys state */
+
+/* Lock key active bits.  Chosen to be equal to the keyboard LED bits. */
+#define SCROLL_LOCK	0x01
+#define NUM_LOCK	0x02
+#define CAPS_LOCK	0x04
 
 PRIVATE char numpad_map[] =
 		{'H', 'Y', 'A', 'B', 'D', 'C', 'V', 'U', 'G', 'S', 'T', '@'};
 
-/* Keyboard structure, 1 per console. */
-struct kb_s {
-  char *ihead;			/* next free spot in input buffer */
-  char *itail;			/* scan code to return to TTY */
-  int icount;			/* # codes in buffer */
-  char ibuf[KB_IN_BYTES];	/* input buffer */
-};
-
-PRIVATE struct kb_s kb_lines[NR_CONS];
-
 FORWARD _PROTOTYPE( int kb_ack, (void) );
 FORWARD _PROTOTYPE( int kb_wait, (void) );
 FORWARD _PROTOTYPE( int func_key, (int scode) );
@@ -92,7 +96,7 @@ int scode;
 {
 /* Map a scan code to an ASCII code. */
 
-  int caps, column;
+  int caps, column, lk;
   u16_t *keyrow;
 
   if (scode == SLASH_SCAN && esc) return '/';	/* don't map numeric slash */
@@ -100,17 +104,18 @@ int scode;
   keyrow = &keymap[scode * MAP_COLS];
 
   caps = shift;
-  if (numlock && HOME_SCAN <= scode && scode <= DEL_SCAN) caps = !caps;
-  if (capslock && (keyrow[0] & HASCAPS)) caps = !caps;
+  lk = locks[current];
+  if ((lk & NUM_LOCK) && HOME_SCAN <= scode && scode <= DEL_SCAN) caps = !caps;
+  if ((lk & CAPS_LOCK) && (keyrow[0] & HASCAPS)) caps = !caps;
 
-  if (alt1 || alt2) {
+  if (alt) {
 	column = 2;
-	if (control || alt2) column = 3;	/* Ctrl + Alt1 == Alt2 */
+	if (ctrl || alt_r) column = 3;	/* Ctrl + Alt == AltGr */
 	if (caps) column = 4;
   } else {
 	column = 0;
 	if (caps) column = 1;
-	if (control) column = 5;
+	if (ctrl) column = 5;
   }
   return keyrow[column] & ~HASCAPS;
 }
@@ -124,36 +129,19 @@ int irq;
 {
 /* A keyboard interrupt has occurred.  Process it. */
 
-  int code;
-  unsigned km;
-  register struct kb_s *kb;
+  int scode;
 
   /* Fetch the character from the keyboard hardware and acknowledge it. */
-  code = scan_keyboard();
+  scode = scan_keyboard();
 
-  /* The IBM keyboard interrupts twice per key, once when depressed, once when
-   * released.  Filter out the latter, ignoring all but the shift-type keys.
-   * The shift-type keys 29, 42, 54, 56, 58, and 69 must be processed normally.
-   */
-
-  if (code & 0200) {
-	/* A key has been released (high bit is set). */
-	km = map_key0(code & 0177);
-	if (km != CTRL && km != SHIFT && km != ALT && km != CALOCK
-			&& km != NLOCK && km != SLOCK && km != EXTKEY)
-		return 1;
-  }
-
-  /* Store the character in memory so the task can get at it later. */
-  kb = kb_addr();
-  if (kb->icount < KB_IN_BYTES) {
-	*kb->ihead++ = code;
-	if (kb->ihead == kb->ibuf + KB_IN_BYTES) kb->ihead = kb->ibuf;
-	kb->icount++;
+  /* Store the scancode in memory so the task can get at it later. */
+  if (icount < KB_IN_BYTES) {
+	*ihead++ = scode;
+	if (ihead == ibuf + KB_IN_BYTES) ihead = ibuf;
+	icount++;
 	tty_table[current].tty_events = 1;
 	force_timeout();
   }
-  /* Else it doesn't fit - discard it. */
   return 1;	/* Reenable keyboard interrupt */
 }
 
@@ -166,19 +154,17 @@ tty_t *tp;
 {
 /* Process characters from the circular keyboard buffer. */
 
-  struct kb_s *kb;
   char buf[3];
   int scode;
   unsigned ch;
 
-  kb = kb_addr();
   tp = &tty_table[current];		/* always use the current console */
 
-  while (kb->icount > 0) {
-	scode = *kb->itail++;			/* take one key scan code */
-	if (kb->itail == kb->ibuf + KB_IN_BYTES) kb->itail = kb->ibuf;
+  while (icount > 0) {
+	scode = *itail++;			/* take one key scan code */
+	if (itail == ibuf + KB_IN_BYTES) itail = ibuf;
 	lock();
-	kb->icount--;
+	icount--;
 	unlock();
 
 	/* Function keys are being used for debug dumps. */
@@ -202,14 +188,17 @@ tty_t *tp;
 	if (ch == ALEFT) {
 		/* Choose lower numbered console as current console. */
 		select_console(current - 1);
+		set_leds();
 	} else
 	if (ch == ARIGHT) {
 		/* Choose higher numbered console as current console. */
 		select_console(current + 1);
+		set_leds();
 	} else
 	if (AF1 <= ch && ch <= AF12) {
 		/* Alt-F1 is console, Alt-F2 is ttyc1, etc. */
 		select_console(ch - AF1);
+		set_leds();
 	}
   }
 }
@@ -225,14 +214,14 @@ int scode;			/* scan code of key just struck or released */
  * as well as keyboards that interrupt on key depression and key release.
  * For efficiency, the interrupt routine filters out most key releases.
  */
-  int ch, make;
+  int ch, make, escape;
   static int CAD_count = 0;
 
   /* Check for CTRL-ALT-DEL, and if found, halt the computer. This would
    * be better done in keyboard() in case TTY is hung, except control and
    * alt are set in the high level code.
    */
-  if (control && (alt1 || alt2) && scode == DEL_SCAN)
+  if (ctrl && alt && (scode == DEL_SCAN || scode == INS_SCAN))
   {
 	if (++CAD_count == 3) wreboot(RBT_HALT);
 	cause_sig(INIT_PROC_NR, SIGABRT);
@@ -240,59 +229,56 @@ int scode;			/* scan code of key just struck or released */
   }
 
   /* High-order bit set on key release. */
-  make = (scode & 0200 ? 0 : 1);	/* 0 = release, 1 = press */
+  make = (scode & 0200) == 0;		/* true if pressed */
 
-  ch = map_key(scode & 0177);		/* map to ASCII */
+  ch = map_key(scode &= 0177);		/* map to ASCII */
+
+  escape = esc;		/* Key is escaped?  (true if added since the XT) */
+  esc = 0;
 
   switch (ch) {
-  	case CTRL:
-		control = make;
-		ch = -1;
+  	case CTRL:		/* Left or right control key */
+		*(escape ? &ctrl_r : &ctrl_l) = make;
+		ctrl = ctrl_l | ctrl_r;
 		break;
-  	case SHIFT:
-		shift = make;
-		ch = -1;
+  	case SHIFT:		/* Left or right shift key */
+		*(scode == RSHIFT_SCAN ? &shift_r : &shift_l) = make;
+		shift = shift_l | shift_r;
 		break;
-  	case ALT:
-		if (make) {
-			if (esc) alt2 = 1; else alt1 = 1;
-		} else {
-			alt1 = alt2 = 0;
-		}
-		ch = -1;
+  	case ALT:		/* Left or right alt key */
+		*(escape ? &alt_r : &alt_l) = make;
+		alt = alt_l | alt_r;
 		break;
-  	case CALOCK:
-		if (make && caps_off) {
-			capslock = 1 - capslock;
+  	case CALOCK:		/* Caps lock - toggle on 0 -> 1 transition */
+		if (caps_down < make) {
+			locks[current] ^= CAPS_LOCK;
 			set_leds();
 		}
-		caps_off = 1 - make;
-		ch = -1;
+		caps_down = make;
 		break;
-  	case NLOCK:
-		if (make && num_off) {
-			numlock = 1 - numlock;
+  	case NLOCK:		/* Num lock */
+		if (num_down < make) {
+			locks[current] ^= NUM_LOCK;
 			set_leds();
 		}
-		num_off = 1 - make;
-		ch = -1;
+		num_down = make;
 		break;
-  	case SLOCK:
-		if (make & slock_off) {
-			slock = 1 - slock;
+  	case SLOCK:		/* Scroll lock */
+		if (scroll_down < make) {
+			locks[current] ^= SCROLL_LOCK;
 			set_leds();
 		}
-		slock_off = 1 - make;
-		ch = -1;
+		scroll_down = make;
 		break;
-  	case EXTKEY:
-		esc = 1;
+  	case EXTKEY:		/* Escape keycode */
+		esc = 1;		/* Next key is escaped */
 		return(-1);
-  	default:
-		if (!make) ch = -1;
+  	default:		/* A normal key */
+		if (make) return(ch);
   }
-  esc = 0;
-  return(ch);
+
+  /* Key release, or a shift type key. */
+  return(-1);
 }
 
 
@@ -301,21 +287,16 @@ int scode;			/* scan code of key just struck or released */
  *===========================================================================*/
 PRIVATE void set_leds()
 {
-/* Set the LEDs on the caps lock and num lock keys */
-
-  unsigned leds;
+/* Set the LEDs on the caps, num, and scroll lock keys */
 
   if (!pc_at) return;	/* PC/XT doesn't have LEDs */
 
-  /* encode LED bits */
-  leds = (slock << 0) | (numlock << 1) | (capslock << 2);
-
   kb_wait();			/* wait for buffer empty  */
   out_byte(KEYBD, LED_CODE);	/* prepare keyboard to accept LED values */
   kb_ack();			/* wait for ack response  */
 
   kb_wait();			/* wait for buffer empty  */
-  out_byte(KEYBD, leds);	/* give keyboard LED values */
+  out_byte(KEYBD, locks[current]); /* give keyboard LED values */
   kb_ack();			/* wait for ack response  */
 }
 
@@ -361,28 +342,15 @@ tty_t *tp;
 {
 /* Initialize the keyboard driver. */
 
-  register struct kb_s *kb;
+  tp->tty_devread = kb_read;	/* Input function */
 
-  /* Input function. */
-  tp->tty_devread = kb_read;
+  set_leds();			/* Turn off numlock led */
 
-  kb = kb_addr();
+  scan_keyboard();		/* Discard leftover keystroke */
 
-  /* Set up input queue. */
-  kb->ihead = kb->itail = kb->ibuf;
-
-  /* Set initial values. */
-  caps_off = 1;
-  num_off = 1;
-  slock_off = 1;
-  esc = 0;
-
-  set_leds();			/* turn off numlock led */
-
-  scan_keyboard();		/* stop lockup from leftover keystroke */
-
-  put_irq_handler(KEYBOARD_IRQ, kbd_hw_int);	/* set the interrupt handler */
-  enable_irq(KEYBOARD_IRQ);	/* safe now everything initialised! */
+  /* Set interrupt handler and enable keyboard IRQ. */
+  put_irq_handler(KEYBOARD_IRQ, kbd_hw_int);
+  enable_irq(KEYBOARD_IRQ);
 }
 
 
@@ -409,6 +377,7 @@ int scode;			/* scan code for a function key */
 
   unsigned code;
 
+  if (scode & 0200) return(FALSE);		/* key release */
   code = map_key0(scode);			/* first ignore modifiers */
   if (code < F1 || code > F12) return(FALSE);	/* not our job */
 
@@ -418,7 +387,7 @@ int scode;			/* scan code for a function key */
   case F2:	map_dmp(); break;	/* print memory map */
   case F3:	toggle_scroll(); break;	/* hardware vs. software scrolling */
 
-#if ENABLE_NETWORKING
+#if ENABLE_DP8390
   case F5:	dp_dump(); break;		/* network statistics */
 #endif
   case CF7:	sigchar(&tty_table[CONSOLE], SIGQUIT); break;
@@ -465,7 +434,7 @@ int how;		/* 0 = halt, 1 = reboot, 2 = panic!, ... */
 
   /* Tell several tasks to stop. */
   cons_stop();
-#if ENABLE_NETWORKING
+#if ENABLE_DP8390
   dp8390_stop();
 #endif
   floppy_stop();
@@ -508,10 +477,10 @@ int how;		/* 0 = halt, 1 = reboot, 2 = panic!, ... */
 
 	/* Return to the boot monitor. */
 	if (how == RBT_HALT) {
-		reboot_code = vir2phys("");
+		phys_copy(vir2phys(""), mon_params, 2);
 	} else
 	if (how == RBT_REBOOT) {
-		reboot_code = vir2phys("delay;boot");
+		phys_copy(vir2phys("delay;boot"), mon_params, 11);
 	}
 	level0(monitor);
   }
