@@ -13,21 +13,15 @@ struct proc {
 
 #if (CHIP == INTEL)
   reg_t p_ldt_sel;		/* selector in gdt giving ldt base and limit*/
-  struct segdesc_s p_ldt[2];	/* local descriptors for code and data */
-				/* 2 is LDT_SIZE - avoid include protect.h */
+  struct segdesc_s p_ldt[4];	/* local descriptors for code and data */
+				/* 4 is LDT_SIZE - avoid include protect.h */
 #endif /* (CHIP == INTEL) */
 
 #if (CHIP == M68000)
   reg_t p_splow;		/* lowest observed stack value */
   int p_trap;			/* trap type (only low byte) */
-#if (SHADOWING == 0)
   char *p_crp;			/* mmu table pointer (really struct _rpr *) */
-#else
-  phys_clicks p_shadow;		/* set if shadowed process image */
-  int align;			/* make the struct size a multiple of 4 */
-#endif
   int p_nflips;			/* statistics */
-  char p_physio;		/* cannot be (un)shadowed now if set */
 #if defined(FPP)
   struct fsave p_fsave;		/* FPP state frame and registers */
   int align2;			/* make the struct size a multiple of 4 */
@@ -38,13 +32,14 @@ struct proc {
 
   int p_nr;			/* number of this process (for fast access) */
 
-  int p_int_blocked;		/* nonzero if int msg blocked by busy task */
-  int p_int_held;		/* nonzero if int msg held by busy syscall */
+  char p_int_blocked;		/* nonzero if int msg blocked by busy task */
+  char p_int_held;		/* nonzero if int msg held by busy syscall */
   struct proc *p_nextheld;	/* next in chain of held-up int processes */
 
-  int p_flags;			/* P_SLOT_FREE, SENDING, RECEIVING, etc. */
+  int p_flags;			/* SENDING, RECEIVING, etc. */
   struct mem_map p_map[NR_SEGS];/* memory map */
   pid_t p_pid;			/* process id passed in from MM */
+  int p_priority;		/* task, server, or user process */
 
   clock_t user_time;		/* user time in ticks */
   clock_t sys_time;		/* sys time in ticks */
@@ -69,13 +64,19 @@ struct proc {
 #define STACK_GUARD	((reg_t) (sizeof(reg_t) == 2 ? 0xBEEF : 0xDEADBEEF))
 
 /* Bits for p_flags in proc[].  A process is runnable iff p_flags == 0. */
-#define P_SLOT_FREE      001	/* set when slot is not in use */
-#define NO_MAP           002	/* keeps unmapped forked child from running */
-#define SENDING          004	/* set when process blocked trying to send */
-#define RECEIVING        010	/* set when process blocked trying to recv */
-#define PENDING          020	/* set when inform() of signal pending */
-#define SIG_PENDING      040	/* keeps to-be-signalled proc from running */
-#define P_STOP		0100	/* set when process is being traced */
+#define NO_MAP		0x01	/* keeps unmapped forked child from running */
+#define SENDING		0x02	/* set when process blocked trying to send */
+#define RECEIVING	0x04	/* set when process blocked trying to recv */
+#define PENDING		0x08	/* set when inform() of signal pending */
+#define SIG_PENDING	0x10	/* keeps to-be-signalled proc from running */
+#define P_STOP		0x20	/* set when process is being traced */
+
+/* Values for p_priority */
+#define PPRI_NONE	0	/* Slot is not in use */
+#define PPRI_TASK	1	/* Part of the kernel */
+#define PPRI_SERVER	2	/* System process outside the kernel */
+#define PPRI_USER	3	/* User process */
+#define PPRI_IDLE	4	/* Idle process */
 
 /* Magic process table addresses. */
 #define BEG_PROC_ADDR (&proc[0])
@@ -88,21 +89,18 @@ struct proc {
 #define isidlehardware(n) ((n) == IDLE || (n) == HARDWARE)
 #define isokprocn(n)      ((unsigned) ((n) + NR_TASKS) < NR_PROCS + NR_TASKS)
 #define isoksrc_dest(n)   (isokprocn(n) || (n) == ANY)
-#define isoksusern(n)     ((unsigned) (n) < NR_PROCS)
-#define isokusern(n)      ((unsigned) ((n) - LOW_USER) < NR_PROCS - LOW_USER)
 #define isrxhardware(n)   ((n) == ANY || (n) == HARDWARE)
 #define issysentn(n)      ((n) == FS_PROC_NR || (n) == MM_PROC_NR)
-#define istaskp(p)        ((p) < END_TASK_ADDR && (p) != proc_addr(IDLE))
-#define isuserp(p)        ((p) >= BEG_USER_ADDR)
+#define isemptyp(p)       ((p)->p_priority == PPRI_NONE)
+#define istaskp(p)        ((p)->p_priority == PPRI_TASK)
+#define isservp(p)        ((p)->p_priority == PPRI_SERVER)
+#define isuserp(p)        ((p)->p_priority == PPRI_USER)
 #define proc_addr(n)      (pproc_addr + NR_TASKS)[(n)]
 #define cproc_addr(n)     (&(proc + NR_TASKS)[(n)])
 #define proc_number(p)    ((p)->p_nr)
 #define proc_vir2phys(p, vir) \
 			  (((phys_bytes)(p)->p_map[D].mem_phys << CLICK_SHIFT) \
 							+ (vir_bytes) (vir))
-#if (SHADOWING == 1)
-#define isshadowp(p)      ((p)->p_shadow != 0)
-#endif
 
 EXTERN struct proc proc[NR_TASKS + NR_PROCS];	/* process table */
 EXTERN struct proc *pproc_addr[NR_TASKS + NR_PROCS];
