@@ -26,10 +26,12 @@
 #define _TABLE
 
 #include "kernel.h"
+#include <stdlib.h>
 #include <termios.h>
 #include <minix/com.h>
 #include "proc.h"
 #include "tty.h"
+#include <ibm/int86.h>
 
 /* The startup routine of each task is given below, from -NR_TASKS upwards.
  * The order of the names here MUST agree with the numerical values assigned to
@@ -40,7 +42,7 @@
 #define	TTY_STACK	(3 * SMALL_STACK)
 #define SYN_ALRM_STACK	SMALL_STACK
 
-#define DP8390_STACK	(SMALL_STACK * ENABLE_NETWORKING)
+#define DP8390_STACK	(SMALL_STACK * ENABLE_DP8390)
 
 #if (CHIP == INTEL)
 #define	IDLE_STACK	((3+3+4) * sizeof(char *))  /* 3 intr, 3 temps, 4 db */
@@ -48,26 +50,12 @@
 #define IDLE_STACK	SMALL_STACK
 #endif
 
-#define	PRINTER_STACK	SMALL_STACK
+#define	PRINTER_STACK	(SMALL_STACK * ENABLE_PRINTER)
 
-#if (CHIP == INTEL)
-#define	WINCH_STACK	(2 * SMALL_STACK * ENABLE_WINI)
-#else
-#define	WINCH_STACK	(3 * SMALL_STACK * ENABLE_WINI)
-#endif
+#define	CTRLR_STACK	(2 * SMALL_STACK)
 
-#if (MACHINE == ATARI)
-#define	SCSI_STACK	(3 * SMALL_STACK)
-#endif
-
-#if (MACHINE == IBM_PC)
-#define	SCSI_STACK	(2 * SMALL_STACK * ENABLE_SCSI)
-#endif
-
-#define CDROM_STACK	(4 * SMALL_STACK * ENABLE_CDROM)
-#define DOSDSK_STACK	(3 * SMALL_STACK * ENABLE_DOSDSK)
-#define AUDIO_STACK	(4 * SMALL_STACK * ENABLE_AUDIO)
-#define MIXER_STACK	(4 * SMALL_STACK * ENABLE_AUDIO)
+#define SB16_STACK	(4 * SMALL_STACK * ENABLE_SB16)
+#define SB16MIXER_STACK	(4 * SMALL_STACK * ENABLE_SB16)
 
 #define	FLOP_STACK	(3 * SMALL_STACK)
 #define	MEM_STACK	SMALL_STACK
@@ -76,38 +64,10 @@
 #define	HARDWARE_STACK	0		/* dummy task, uses kernel stack */
 
 
-#define	TOT_STACK_SPACE		(TTY_STACK + DP8390_STACK + SCSI_STACK + \
+#define	TOT_STACK_SPACE		(TTY_STACK + DP8390_STACK + \
 	SYN_ALRM_STACK + IDLE_STACK + HARDWARE_STACK + PRINTER_STACK + \
-	WINCH_STACK + FLOP_STACK + MEM_STACK + CLOCK_STACK + SYS_STACK + \
-	CDROM_STACK + DOSDSK_STACK + AUDIO_STACK + MIXER_STACK)
-
-
-/* Default choices for tasks with more than one driver. */
-#if ENABLE_AT_WINI
-#define winchester_task	at_winchester_task
-#elif ENABLE_BIOS_WINI
-#define winchester_task	bios_winchester_task
-#elif ENABLE_ESDI_WINI
-#define winchester_task	esdi_winchester_task
-#elif ENABLE_XT_WINI
-#define winchester_task	xt_winchester_task
-#endif
-
-#if ENABLE_AHA1540_SCSI
-#define scsi_task	aha1540_scsi_task
-#endif
-
-#if ENABLE_DOSFILE
-#define dosdsk_task	dosfile_task
-#elif ENABLE_DOSFAT
-#define dosdsk_task	dosfat_task
-#endif
-
-/* CDROM and AUDIO may in the future have different choices like
- * WINCHESTER, but for now the choice is fixed.
- */
-#define cdrom_task	mcd_task
-#define audio_task	dsp_task
+	NR_CTRLRS * CTRLR_STACK + FLOP_STACK + MEM_STACK + CLOCK_STACK + \
+	SYS_STACK + SB16_STACK + SB16MIXER_STACK)
 
 
 /*
@@ -120,28 +80,30 @@
 
 PUBLIC struct tasktab tasktab[] = {
 	{ tty_task,		TTY_STACK,	"TTY"		},
-#if ENABLE_NETWORKING
+#if ENABLE_DP8390
 	{ dp8390_task,		DP8390_STACK,	"DP8390"	},
 #endif
-#if ENABLE_DOSDSK
-	{ dosdsk_task,		DOSDSK_STACK,	"DOSDSK"	},
+#if ENABLE_SB16
+	{ sb16_task,		SB16_STACK,	"SB16"		},
+	{ sb16mixer_task,	SB16MIXER_STACK,"SB16MIX"	},
 #endif
-#if ENABLE_CDROM
-	{ cdrom_task,		CDROM_STACK,	"CDROM"		},
+#if ENABLE_PRINTER
+	{ printer_task,		PRINTER_STACK,	"PRINTER"	},
 #endif
-#if ENABLE_AUDIO
-	{ audio_task,		AUDIO_STACK,	"AUDIO"		},
-	{ mixer_task,		MIXER_STACK,	"MIXER"		},
+#if NR_CTRLRS >= 4
+	{ nop_task,		CTRLR_STACK,	"(C3)"		},
 #endif
-#if ENABLE_SCSI
-	{ scsi_task,		SCSI_STACK,	"SCSI"		},
+#if NR_CTRLRS >= 3
+	{ nop_task,		CTRLR_STACK,	"(C2)"		},
 #endif
-#if ENABLE_WINI
-	{ winchester_task,	WINCH_STACK,	"WINCH"		},
+#if NR_CTRLRS >= 2
+	{ nop_task,		CTRLR_STACK,	"(C1)"		},
+#endif
+#if NR_CTRLRS >= 1
+	{ nop_task,		CTRLR_STACK,	"(C0)"		},
 #endif
 	{ syn_alrm_task,	SYN_ALRM_STACK, "SYN_AL"	},
 	{ idle_task,		IDLE_STACK,	"IDLE"		},
-	{ printer_task,		PRINTER_STACK,	"PRINTER"	},
 	{ floppy_task,		FLOP_STACK,	"FLOPPY"	},
 	{ mem_task,		MEM_STACK,	"MEMORY"	},
 	{ clock_task,		CLOCK_STACK,	"CLOCK"		},
@@ -149,15 +111,12 @@ PUBLIC struct tasktab tasktab[] = {
 	{ 0,			HARDWARE_STACK,	"HARDWAR"	},
 	{ 0,			0,		"MM"		},
 	{ 0,			0,		"FS"		},
-#if ENABLE_NETWORKING
-	{ 0,			0,		"INET"		},
-#endif
 	{ 0,			0,		"INIT"		},
 };
 
 /* Mapping from driver names to driver functions, e.g. "bios" -> bios_wini. */
 PRIVATE struct drivertab {
-	char drivername[6];
+	char drivername[8];
 	task_t *driver;
 } drivertab[] = {
 
@@ -178,35 +137,15 @@ PRIVATE struct drivertab {
 #endif
 
 #if ENABLE_AHA1540_SCSI
-	{ "a1540",	aha1540_scsi_task	},
+	{ "aha1540",	aha1540_scsi_task	},
 #endif
 
 #if ENABLE_DOSFILE
-	{ "file",	dosfile_task		},
+	{ "dosfile",	dosfile_task		},
 #endif
 
-#if ENABLE_DOSFAT
-	{ "fat",	dosfat_task		},
-#endif
-
-};
-
-/* Mapping from environment variables to tasks, e.g. "hd" -> WINCHESTER. */
-PRIVATE struct envtab {
-	char env[5];
-	char taskno;
-} envtab[] = {
-
-#if ENABLE_WINI
-	{ "hd",		WINCHESTER	},
-#endif
-
-#if ENABLE_SCSI
-	{ "sd",		SCSI		},
-#endif
-
-#if ENABLE_DOSDSK
-	{ "dosd",	DOSDSK		},
+#if ENABLE_FATFILE
+	{ "fatfile",	fatfile_task		},
 #endif
 
 };
@@ -216,17 +155,22 @@ PRIVATE struct envtab {
  *===========================================================================*/
 PUBLIC void mapdrivers()
 {
-  /* Select drivers and update the task table to that selection. */
-  struct envtab *ep;
+  /* Map drivers to controllers and update the task table to that selection. */
+  static char name[] = "c0";
+  int c;
   struct drivertab *dp;
   char *drivername;
+  struct tasktab *tp;
 
-  for (ep = envtab; ep < envtab + sizeof(envtab)/sizeof(envtab[0]); ep++) {
-	if ((drivername = k_getenv(ep->env)) == NULL) continue;
+  for (c= 0; c < NR_CTRLRS; c++) {
+	name[1] = '0' + c;
+	if ((drivername = getenv(name)) == NULL) continue;
 	for (dp = drivertab;
 	     dp < drivertab + sizeof(drivertab)/sizeof(drivertab[0]); dp++) {
 		if (strcmp(drivername, dp->drivername) == 0) {
-			tasktab[ep->taskno + NR_TASKS].initial_pc = dp->driver;
+			tp = &tasktab[CTRLR(c) + NR_TASKS];
+			tp->initial_pc = dp->driver;
+			strcpy(tp->name, drivername);
 		}
 	}
   }
