@@ -12,7 +12,7 @@ INIT_ASSERT
 #include "dp8390.h"
 #include "wdeth.h"
 
-#if (ENABLE_NETWORKING && ENABLE_WDETH) || __minix_vmd
+#if (ENABLE_DP8390 && ENABLE_WDETH) || __minix_vmd
 
 #define WET_ETHERNET	0x01		/* Ethernet transceiver */
 #define WET_STARLAN	0x02		/* Starlan transceiver */
@@ -359,7 +359,7 @@ dpeth_t *dep;
 	return tlb == E_TLB_SMC8216C;
 }
 
-#endif /* ENABLE_NETWORKING && ENABLE_WDETH */
+#endif /* ENABLE_DP8390 && ENABLE_WDETH */
 
 /*
  * $PchId: wdeth.c,v 1.6 1996/01/19 22:51:37 philip Exp $
