@@ -3,7 +3,7 @@
   See the copyright notice in the ACK home directory, in the file "Copyright".
 */
 
-/* $Header: mlf8.c,v 1.3 88/07/25 10:44:01 ceriel Exp $ */
+/* $Header: mlf8.c,v 1.4 93/01/05 12:05:44 ceriel Exp $ */
 
 /*
  * Multiply Double Precision Float (MLF 8)
@@ -11,7 +11,7 @@
 
 #include	"FP_types.h"
 
-DOUBLE
+void
 mlf8(s2,s1)
 DOUBLE	s1,s2;
 {
@@ -22,5 +22,4 @@ DOUBLE	s1,s2;
 		/* do a multiply */
 	mul_ext(&e1,&e2);
 	compact(&e1,&s1.d[0],sizeof(DOUBLE));
-	return(s1);
 }
