@@ -213,12 +213,7 @@ void test20b()
   if (errno != ENOTDIR) e(12);
   if ((dirp = opendir(".")) == DIR_NULL) e(13);
   if (closedir(dirp) != 0) e(14);
-  if (closedir(dirp) >= 0) e(15);
-  if (readdir(dirp) != (struct dirent *) NULL) e(16);
-  if (errno != EBADF) e(17);
-  if (readdir(dirp - 2) != (struct dirent *) NULL) e(18);
-  if (errno != EBADF) e(19);
-  if (unlink("abc") != 0) e(20);
+  if (unlink("abc") != 0) e(15);
 
 }
 
