@@ -23,7 +23,7 @@
 
 int errct = 0;
 int subtest = 1;
-int superuser;
+int superuser;			/* nonzero if uid == euid (euid == 0 always) */
 char MaxName[NAME_MAX + 1];	/* Name of maximum length */
 char MaxPath[PATH_MAX];		/* Same for path */
 char ToLongName[NAME_MAX + 2];	/* Name of maximum +1 length */
@@ -49,23 +49,25 @@ char *argv[];
   if (argc == 2) m = atoi(argv[1]);
   printf("Test 33 ");
   fflush(stdout);
-  umask(0000);
-  System("rm -rf DIR_33; mkdir DIR_33");
 
   if (geteuid() != 0) {
 	printf("must be setuid root; test aborted\n");
 	exit(1);
   }
+  if (getuid() == 0) {
+       printf("must be setuid root logged in as someone else; test aborted\n");
+       exit(1);
+  }
 
+  umask(0000);
+  System("rm -rf DIR_33; mkdir DIR_33");
   Chdir("DIR_33");
   makelongnames();
-  superuser = (geteuid() == 0);
+  superuser = (getuid() == 0);
 
   for (i = 0; i < ITERATIONS; i++) {
-/* Something wrong here.
 	if (m & 0001) test33a();
 	if (m & 0002) test33b();
- */
 	if (m & 0004) test33c();
 	if (m & 0010) test33d();
   }
@@ -77,7 +79,7 @@ void test33a()
   int stat_loc;			/* For the wait(&stat_loc) call. */
 
   subtest = 1;
-  System("/bin/rm -rf ../DIR_33/*");
+  System("rm -rf ../DIR_33/*");
 
   /* To test normal access first make some files for real uid. */
   switch (fork()) {
@@ -183,7 +185,7 @@ void test33b()
   int stat_loc;			/* For the wait(&stat_loc) call. */
 
   subtest = 2;
-  System("/bin/rm -rf ../DIR_33/*");
+  System("rm -rf ../DIR_33/*");
 
   switch (fork()) {
       case -1:	printf("Can't fork\n");	break;
@@ -237,7 +239,7 @@ void test33b()
   }
 
   /* If the real uid != effective uid. */
-  if (getuid() != geteuid() && !superuser) {
+  if (!superuser) {
 	System("rm -rf [_r][_w][_x]");
 	System("> rwx");
 	Chmod("rwx", 0700);
@@ -329,7 +331,7 @@ void test33c()
   int i;
 
   subtest = 3;
-  System("/bin/rm -rf ../DIR_33/*");
+  System("rm -rf ../DIR_33/*");
 
   /* Test what access() does with non existing files. */
   System("rm -rf nonexist");
@@ -351,29 +353,28 @@ void test33c()
   if (errno != ENOENT) e(16);
 
   /* Test access on a nonsearchable path. */
-  System("mkdir nosearch");
-  System("> nosearch/file");
-  System("> file");
-  Chmod("nosearch/file", 05777);
-  Chmod("file", 05777);
-  Chmod("nosearch", 0677);
-  if (!superuser) {
-	if (access("nosearch/file", F_OK) != -1) e(17);
-	if (errno != EACCES) e(18);
-	if (access("nosearch/../file", F_OK) != -1) e(19);
-	if (errno != EACCES) e(20);
-  }
-  if (superuser) {
-	if (access("nosearch/file", F_OK) != 0) e(21);
-  }
-
-  /* Double check the file is accessable. */
-  Chmod("nosearch", 0700);
-  if (access("nosearch/file", F_OK) != 0) e(22);
+  if (mkdir("nosearch", 0777) != 0) e(1000);
+  if ( (i = creat("nosearch/file", 0666)) < 0) e(1001);
+  if (close(i) < 0) e(1002);
+  if ( (i = creat("file", 0666) < 0)) e(1003);
+  if (close(i) < 0) e(1004);
+  if (chmod("nosearch/file", 05777) < 0) e(1005);
+  if (chmod("file", 05777) < 0) e(1006);
+  if (chmod("nosearch", 0677) != 0) e(1007);
+  if (access("nosearch/file", F_OK) != 0) e(17);
 
   /* Test ToLongName and ToLongPath */
+#ifdef _POSIX_NO_TRUNC
+# if _POSIX_NO_TRUNC - 0 != -1
+  if (access(ToLongName, F_OK) != -1) e(23);
+  if (errno != ENAMETOOLONG) e(24);
+# else
   if (close(creat(ToLongName, 0777)) != 0) e(25);
   if (access(ToLongName, F_OK) != 0) e(26);
+# endif
+#else
+# include "error, this case requires dynamic checks and is not handled"
+#endif
   ToLongPath[PATH_MAX - 2] = '/';
   ToLongPath[PATH_MAX - 1] = 'a';
   if (access(ToLongPath, F_OK) != -1) e(27);
@@ -410,7 +411,7 @@ void test33d()
 {				/* Test access() flags. */
 #define EXCLUDE(a,b)	(((a)^(b)) == ((a)|(b)))
   subtest = 4;
-  System("/bin/rm -rf ../DIR_33/*");
+  System("rm -rf ../DIR_33/*");
 
   /* The test are rather strong, stronger that POSIX specifies. */
   /* The should be OR able, this test tests if all the 1 bits */
